//
// Copyright 2002, Lowell Boggs Jr.
//
// This file or directory, containing source code for a computer program,
// is Copyrighted by Lowell Boggs, Jr.  987 Regency Drive, Lewisville
// TX (USA), 75067.  You may use, copy, modify, and distribute this
// source file without charge or obligation so long as you agree to
// the following:
//
//  1.  You must indemnify Lowell Boggs against any and all financial
//      obligations caused by its use, misuse, function, or malfunction.
//      Further, you acknowledge that there is no warranty of any kind,
//      whatsoever.
//
//  2.  You agree not to attempt to patent any portion of this original
//      work -- though you may attempt to patent your own extensions to
//      it if you so choose.
//
//  3.  You keep this copyright notice with the file and all copies
//      of the file and do not change it anyway except language translation.
//
// You are responsible for enforcing your own compliance with these
// conditions and may not use this source file if you cannot agree to the
// above terms and conditions.

///@file
/// Implements members of the UserInfo class.

#include <cxxtls/user.h>
#include <cxxtls/options.h>

namespace cxxtls
{


#ifdef _MSC_VER
# define getuid()  0  /* stubbed out on nt for the moment */
# include <stdlib.h>
static std::string get_environment_value(char const *name)
{
  if(ProgramOptions::global_options_)
    return ProgramOptions::global_options_->getenv(name).value();

  return getenv(name);
}
#else
# include <unistd.h>
#include <pwd.h>
#endif



UserInfo
UserInfo::
get(long uid)
{
  UserInfo rv;

#ifdef _MSC_VER
  std::string username = get_environment_value("USERNAME");

  if(username.empty())
    username="USER";

  std::string shell = get_environment_value("COMSPEC");


  if(shell.empty())
  {
    shell = get_environment_value("ComSpec");

    if(shell.empty())
        shell = "C:\\Windows\\System32\\cmd.exe";
  }


  std::string home = get_environment_value("HOME");

  if(home.empty())
    home = "C:/";

  rv.name_     = username;
  rv.home_dir_ = home;
  rv.shell_    = shell;
  rv.uid_      = 0;
  rv.gid_      = 0;

#else

  struct passwd &pwent = *getpwuid(getuid());

  if( &pwent == 0 )
   return rv;
  else
  {
    rv.name_      = pwent.pw_name;
    rv.password_  = pwent.pw_passwd;
    rv.home_dir_  = pwent.pw_dir;
    rv.shell_     = pwent.pw_shell;
    rv.uid_       = pwent.pw_uid;
    rv.gid_       = pwent.pw_gid;

  }

#endif

  return rv;

}

UserInfo
UserInfo::
get(char const *user)
{
  UserInfo rv;

#ifdef _MSC_VER

  // just return the current user's info

  std::string username = get_environment_value("USERNAME");

  if(username.empty())
    username="USER";

  std::string shell = get_environment_value("COMSPEC");


  if(shell.empty())
    shell = "C:\\WINNT\\system32\\cmd.exe";


  std::string home = get_environment_value("HOME");

  if(home.empty())
    home = "C:/";

  rv.name_     = username;
  rv.home_dir_ = home;
  rv.shell_    = shell;
  rv.uid_      = 0;
  rv.gid_      = 0;

#else

  struct passwd &pwent = * getpwnam(user);


  if( &pwent == 0 )
   return rv;
  else
  {
    rv.name_      = pwent.pw_name;
    rv.password_  = pwent.pw_passwd;
    rv.home_dir_  = pwent.pw_dir;
    rv.shell_     = pwent.pw_shell;
    rv.uid_       = pwent.pw_uid;
    rv.gid_       = pwent.pw_gid;

  }

#endif

  return rv;

}


UserInfo
UserInfo::
get()
{
  return get(long(getuid()));
}

} // namespace cxxtls
